# Generated from Modelica.g4 by ANTLR 4.7
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .ModelicaParser import ModelicaParser
else:
    from ModelicaParser import ModelicaParser

# This class defines a complete listener for a parse tree produced by ModelicaParser.
class ModelicaListener(ParseTreeListener):

    # Enter a parse tree produced by ModelicaParser#stored_definition.
    def enterStored_definition(self, ctx:ModelicaParser.Stored_definitionContext):
        pass

    # Exit a parse tree produced by ModelicaParser#stored_definition.
    def exitStored_definition(self, ctx:ModelicaParser.Stored_definitionContext):
        pass


    # Enter a parse tree produced by ModelicaParser#stored_definition_class.
    def enterStored_definition_class(self, ctx:ModelicaParser.Stored_definition_classContext):
        pass

    # Exit a parse tree produced by ModelicaParser#stored_definition_class.
    def exitStored_definition_class(self, ctx:ModelicaParser.Stored_definition_classContext):
        pass


    # Enter a parse tree produced by ModelicaParser#class_definition.
    def enterClass_definition(self, ctx:ModelicaParser.Class_definitionContext):
        pass

    # Exit a parse tree produced by ModelicaParser#class_definition.
    def exitClass_definition(self, ctx:ModelicaParser.Class_definitionContext):
        pass


    # Enter a parse tree produced by ModelicaParser#class_prefixes.
    def enterClass_prefixes(self, ctx:ModelicaParser.Class_prefixesContext):
        pass

    # Exit a parse tree produced by ModelicaParser#class_prefixes.
    def exitClass_prefixes(self, ctx:ModelicaParser.Class_prefixesContext):
        pass


    # Enter a parse tree produced by ModelicaParser#class_type.
    def enterClass_type(self, ctx:ModelicaParser.Class_typeContext):
        pass

    # Exit a parse tree produced by ModelicaParser#class_type.
    def exitClass_type(self, ctx:ModelicaParser.Class_typeContext):
        pass


    # Enter a parse tree produced by ModelicaParser#class_spec_comp.
    def enterClass_spec_comp(self, ctx:ModelicaParser.Class_spec_compContext):
        pass

    # Exit a parse tree produced by ModelicaParser#class_spec_comp.
    def exitClass_spec_comp(self, ctx:ModelicaParser.Class_spec_compContext):
        pass


    # Enter a parse tree produced by ModelicaParser#class_spec_base.
    def enterClass_spec_base(self, ctx:ModelicaParser.Class_spec_baseContext):
        pass

    # Exit a parse tree produced by ModelicaParser#class_spec_base.
    def exitClass_spec_base(self, ctx:ModelicaParser.Class_spec_baseContext):
        pass


    # Enter a parse tree produced by ModelicaParser#class_spec_enum.
    def enterClass_spec_enum(self, ctx:ModelicaParser.Class_spec_enumContext):
        pass

    # Exit a parse tree produced by ModelicaParser#class_spec_enum.
    def exitClass_spec_enum(self, ctx:ModelicaParser.Class_spec_enumContext):
        pass


    # Enter a parse tree produced by ModelicaParser#class_spec_der.
    def enterClass_spec_der(self, ctx:ModelicaParser.Class_spec_derContext):
        pass

    # Exit a parse tree produced by ModelicaParser#class_spec_der.
    def exitClass_spec_der(self, ctx:ModelicaParser.Class_spec_derContext):
        pass


    # Enter a parse tree produced by ModelicaParser#class_spec_extends.
    def enterClass_spec_extends(self, ctx:ModelicaParser.Class_spec_extendsContext):
        pass

    # Exit a parse tree produced by ModelicaParser#class_spec_extends.
    def exitClass_spec_extends(self, ctx:ModelicaParser.Class_spec_extendsContext):
        pass


    # Enter a parse tree produced by ModelicaParser#base_prefix.
    def enterBase_prefix(self, ctx:ModelicaParser.Base_prefixContext):
        pass

    # Exit a parse tree produced by ModelicaParser#base_prefix.
    def exitBase_prefix(self, ctx:ModelicaParser.Base_prefixContext):
        pass


    # Enter a parse tree produced by ModelicaParser#enum_list.
    def enterEnum_list(self, ctx:ModelicaParser.Enum_listContext):
        pass

    # Exit a parse tree produced by ModelicaParser#enum_list.
    def exitEnum_list(self, ctx:ModelicaParser.Enum_listContext):
        pass


    # Enter a parse tree produced by ModelicaParser#enumeration_literal.
    def enterEnumeration_literal(self, ctx:ModelicaParser.Enumeration_literalContext):
        pass

    # Exit a parse tree produced by ModelicaParser#enumeration_literal.
    def exitEnumeration_literal(self, ctx:ModelicaParser.Enumeration_literalContext):
        pass


    # Enter a parse tree produced by ModelicaParser#composition.
    def enterComposition(self, ctx:ModelicaParser.CompositionContext):
        pass

    # Exit a parse tree produced by ModelicaParser#composition.
    def exitComposition(self, ctx:ModelicaParser.CompositionContext):
        pass


    # Enter a parse tree produced by ModelicaParser#language_specification.
    def enterLanguage_specification(self, ctx:ModelicaParser.Language_specificationContext):
        pass

    # Exit a parse tree produced by ModelicaParser#language_specification.
    def exitLanguage_specification(self, ctx:ModelicaParser.Language_specificationContext):
        pass


    # Enter a parse tree produced by ModelicaParser#external_function_call.
    def enterExternal_function_call(self, ctx:ModelicaParser.External_function_callContext):
        pass

    # Exit a parse tree produced by ModelicaParser#external_function_call.
    def exitExternal_function_call(self, ctx:ModelicaParser.External_function_callContext):
        pass


    # Enter a parse tree produced by ModelicaParser#element_list.
    def enterElement_list(self, ctx:ModelicaParser.Element_listContext):
        pass

    # Exit a parse tree produced by ModelicaParser#element_list.
    def exitElement_list(self, ctx:ModelicaParser.Element_listContext):
        pass


    # Enter a parse tree produced by ModelicaParser#element.
    def enterElement(self, ctx:ModelicaParser.ElementContext):
        pass

    # Exit a parse tree produced by ModelicaParser#element.
    def exitElement(self, ctx:ModelicaParser.ElementContext):
        pass


    # Enter a parse tree produced by ModelicaParser#regular_element.
    def enterRegular_element(self, ctx:ModelicaParser.Regular_elementContext):
        pass

    # Exit a parse tree produced by ModelicaParser#regular_element.
    def exitRegular_element(self, ctx:ModelicaParser.Regular_elementContext):
        pass


    # Enter a parse tree produced by ModelicaParser#replaceable_element.
    def enterReplaceable_element(self, ctx:ModelicaParser.Replaceable_elementContext):
        pass

    # Exit a parse tree produced by ModelicaParser#replaceable_element.
    def exitReplaceable_element(self, ctx:ModelicaParser.Replaceable_elementContext):
        pass


    # Enter a parse tree produced by ModelicaParser#import_clause.
    def enterImport_clause(self, ctx:ModelicaParser.Import_clauseContext):
        pass

    # Exit a parse tree produced by ModelicaParser#import_clause.
    def exitImport_clause(self, ctx:ModelicaParser.Import_clauseContext):
        pass


    # Enter a parse tree produced by ModelicaParser#import_list.
    def enterImport_list(self, ctx:ModelicaParser.Import_listContext):
        pass

    # Exit a parse tree produced by ModelicaParser#import_list.
    def exitImport_list(self, ctx:ModelicaParser.Import_listContext):
        pass


    # Enter a parse tree produced by ModelicaParser#extends_clause.
    def enterExtends_clause(self, ctx:ModelicaParser.Extends_clauseContext):
        pass

    # Exit a parse tree produced by ModelicaParser#extends_clause.
    def exitExtends_clause(self, ctx:ModelicaParser.Extends_clauseContext):
        pass


    # Enter a parse tree produced by ModelicaParser#constraining_clause.
    def enterConstraining_clause(self, ctx:ModelicaParser.Constraining_clauseContext):
        pass

    # Exit a parse tree produced by ModelicaParser#constraining_clause.
    def exitConstraining_clause(self, ctx:ModelicaParser.Constraining_clauseContext):
        pass


    # Enter a parse tree produced by ModelicaParser#component_clause.
    def enterComponent_clause(self, ctx:ModelicaParser.Component_clauseContext):
        pass

    # Exit a parse tree produced by ModelicaParser#component_clause.
    def exitComponent_clause(self, ctx:ModelicaParser.Component_clauseContext):
        pass


    # Enter a parse tree produced by ModelicaParser#type_prefix.
    def enterType_prefix(self, ctx:ModelicaParser.Type_prefixContext):
        pass

    # Exit a parse tree produced by ModelicaParser#type_prefix.
    def exitType_prefix(self, ctx:ModelicaParser.Type_prefixContext):
        pass


    # Enter a parse tree produced by ModelicaParser#type_specifier_element.
    def enterType_specifier_element(self, ctx:ModelicaParser.Type_specifier_elementContext):
        pass

    # Exit a parse tree produced by ModelicaParser#type_specifier_element.
    def exitType_specifier_element(self, ctx:ModelicaParser.Type_specifier_elementContext):
        pass


    # Enter a parse tree produced by ModelicaParser#type_specifier.
    def enterType_specifier(self, ctx:ModelicaParser.Type_specifierContext):
        pass

    # Exit a parse tree produced by ModelicaParser#type_specifier.
    def exitType_specifier(self, ctx:ModelicaParser.Type_specifierContext):
        pass


    # Enter a parse tree produced by ModelicaParser#component_list.
    def enterComponent_list(self, ctx:ModelicaParser.Component_listContext):
        pass

    # Exit a parse tree produced by ModelicaParser#component_list.
    def exitComponent_list(self, ctx:ModelicaParser.Component_listContext):
        pass


    # Enter a parse tree produced by ModelicaParser#component_declaration.
    def enterComponent_declaration(self, ctx:ModelicaParser.Component_declarationContext):
        pass

    # Exit a parse tree produced by ModelicaParser#component_declaration.
    def exitComponent_declaration(self, ctx:ModelicaParser.Component_declarationContext):
        pass


    # Enter a parse tree produced by ModelicaParser#condition_attribute.
    def enterCondition_attribute(self, ctx:ModelicaParser.Condition_attributeContext):
        pass

    # Exit a parse tree produced by ModelicaParser#condition_attribute.
    def exitCondition_attribute(self, ctx:ModelicaParser.Condition_attributeContext):
        pass


    # Enter a parse tree produced by ModelicaParser#declaration.
    def enterDeclaration(self, ctx:ModelicaParser.DeclarationContext):
        pass

    # Exit a parse tree produced by ModelicaParser#declaration.
    def exitDeclaration(self, ctx:ModelicaParser.DeclarationContext):
        pass


    # Enter a parse tree produced by ModelicaParser#modification_class.
    def enterModification_class(self, ctx:ModelicaParser.Modification_classContext):
        pass

    # Exit a parse tree produced by ModelicaParser#modification_class.
    def exitModification_class(self, ctx:ModelicaParser.Modification_classContext):
        pass


    # Enter a parse tree produced by ModelicaParser#modification_assignment.
    def enterModification_assignment(self, ctx:ModelicaParser.Modification_assignmentContext):
        pass

    # Exit a parse tree produced by ModelicaParser#modification_assignment.
    def exitModification_assignment(self, ctx:ModelicaParser.Modification_assignmentContext):
        pass


    # Enter a parse tree produced by ModelicaParser#modification_assignment2.
    def enterModification_assignment2(self, ctx:ModelicaParser.Modification_assignment2Context):
        pass

    # Exit a parse tree produced by ModelicaParser#modification_assignment2.
    def exitModification_assignment2(self, ctx:ModelicaParser.Modification_assignment2Context):
        pass


    # Enter a parse tree produced by ModelicaParser#class_modification.
    def enterClass_modification(self, ctx:ModelicaParser.Class_modificationContext):
        pass

    # Exit a parse tree produced by ModelicaParser#class_modification.
    def exitClass_modification(self, ctx:ModelicaParser.Class_modificationContext):
        pass


    # Enter a parse tree produced by ModelicaParser#argument_list.
    def enterArgument_list(self, ctx:ModelicaParser.Argument_listContext):
        pass

    # Exit a parse tree produced by ModelicaParser#argument_list.
    def exitArgument_list(self, ctx:ModelicaParser.Argument_listContext):
        pass


    # Enter a parse tree produced by ModelicaParser#argument.
    def enterArgument(self, ctx:ModelicaParser.ArgumentContext):
        pass

    # Exit a parse tree produced by ModelicaParser#argument.
    def exitArgument(self, ctx:ModelicaParser.ArgumentContext):
        pass


    # Enter a parse tree produced by ModelicaParser#element_modification_or_replaceable.
    def enterElement_modification_or_replaceable(self, ctx:ModelicaParser.Element_modification_or_replaceableContext):
        pass

    # Exit a parse tree produced by ModelicaParser#element_modification_or_replaceable.
    def exitElement_modification_or_replaceable(self, ctx:ModelicaParser.Element_modification_or_replaceableContext):
        pass


    # Enter a parse tree produced by ModelicaParser#element_modification.
    def enterElement_modification(self, ctx:ModelicaParser.Element_modificationContext):
        pass

    # Exit a parse tree produced by ModelicaParser#element_modification.
    def exitElement_modification(self, ctx:ModelicaParser.Element_modificationContext):
        pass


    # Enter a parse tree produced by ModelicaParser#element_redeclaration.
    def enterElement_redeclaration(self, ctx:ModelicaParser.Element_redeclarationContext):
        pass

    # Exit a parse tree produced by ModelicaParser#element_redeclaration.
    def exitElement_redeclaration(self, ctx:ModelicaParser.Element_redeclarationContext):
        pass


    # Enter a parse tree produced by ModelicaParser#element_replaceable.
    def enterElement_replaceable(self, ctx:ModelicaParser.Element_replaceableContext):
        pass

    # Exit a parse tree produced by ModelicaParser#element_replaceable.
    def exitElement_replaceable(self, ctx:ModelicaParser.Element_replaceableContext):
        pass


    # Enter a parse tree produced by ModelicaParser#component_clause1.
    def enterComponent_clause1(self, ctx:ModelicaParser.Component_clause1Context):
        pass

    # Exit a parse tree produced by ModelicaParser#component_clause1.
    def exitComponent_clause1(self, ctx:ModelicaParser.Component_clause1Context):
        pass


    # Enter a parse tree produced by ModelicaParser#component_declaration1.
    def enterComponent_declaration1(self, ctx:ModelicaParser.Component_declaration1Context):
        pass

    # Exit a parse tree produced by ModelicaParser#component_declaration1.
    def exitComponent_declaration1(self, ctx:ModelicaParser.Component_declaration1Context):
        pass


    # Enter a parse tree produced by ModelicaParser#short_class_definition.
    def enterShort_class_definition(self, ctx:ModelicaParser.Short_class_definitionContext):
        pass

    # Exit a parse tree produced by ModelicaParser#short_class_definition.
    def exitShort_class_definition(self, ctx:ModelicaParser.Short_class_definitionContext):
        pass


    # Enter a parse tree produced by ModelicaParser#equation_block.
    def enterEquation_block(self, ctx:ModelicaParser.Equation_blockContext):
        pass

    # Exit a parse tree produced by ModelicaParser#equation_block.
    def exitEquation_block(self, ctx:ModelicaParser.Equation_blockContext):
        pass


    # Enter a parse tree produced by ModelicaParser#equation_section.
    def enterEquation_section(self, ctx:ModelicaParser.Equation_sectionContext):
        pass

    # Exit a parse tree produced by ModelicaParser#equation_section.
    def exitEquation_section(self, ctx:ModelicaParser.Equation_sectionContext):
        pass


    # Enter a parse tree produced by ModelicaParser#statement_block.
    def enterStatement_block(self, ctx:ModelicaParser.Statement_blockContext):
        pass

    # Exit a parse tree produced by ModelicaParser#statement_block.
    def exitStatement_block(self, ctx:ModelicaParser.Statement_blockContext):
        pass


    # Enter a parse tree produced by ModelicaParser#algorithm_section.
    def enterAlgorithm_section(self, ctx:ModelicaParser.Algorithm_sectionContext):
        pass

    # Exit a parse tree produced by ModelicaParser#algorithm_section.
    def exitAlgorithm_section(self, ctx:ModelicaParser.Algorithm_sectionContext):
        pass


    # Enter a parse tree produced by ModelicaParser#equation_simple.
    def enterEquation_simple(self, ctx:ModelicaParser.Equation_simpleContext):
        pass

    # Exit a parse tree produced by ModelicaParser#equation_simple.
    def exitEquation_simple(self, ctx:ModelicaParser.Equation_simpleContext):
        pass


    # Enter a parse tree produced by ModelicaParser#equation_if.
    def enterEquation_if(self, ctx:ModelicaParser.Equation_ifContext):
        pass

    # Exit a parse tree produced by ModelicaParser#equation_if.
    def exitEquation_if(self, ctx:ModelicaParser.Equation_ifContext):
        pass


    # Enter a parse tree produced by ModelicaParser#equation_for.
    def enterEquation_for(self, ctx:ModelicaParser.Equation_forContext):
        pass

    # Exit a parse tree produced by ModelicaParser#equation_for.
    def exitEquation_for(self, ctx:ModelicaParser.Equation_forContext):
        pass


    # Enter a parse tree produced by ModelicaParser#equation_connect_clause.
    def enterEquation_connect_clause(self, ctx:ModelicaParser.Equation_connect_clauseContext):
        pass

    # Exit a parse tree produced by ModelicaParser#equation_connect_clause.
    def exitEquation_connect_clause(self, ctx:ModelicaParser.Equation_connect_clauseContext):
        pass


    # Enter a parse tree produced by ModelicaParser#equation_when.
    def enterEquation_when(self, ctx:ModelicaParser.Equation_whenContext):
        pass

    # Exit a parse tree produced by ModelicaParser#equation_when.
    def exitEquation_when(self, ctx:ModelicaParser.Equation_whenContext):
        pass


    # Enter a parse tree produced by ModelicaParser#equation_function.
    def enterEquation_function(self, ctx:ModelicaParser.Equation_functionContext):
        pass

    # Exit a parse tree produced by ModelicaParser#equation_function.
    def exitEquation_function(self, ctx:ModelicaParser.Equation_functionContext):
        pass


    # Enter a parse tree produced by ModelicaParser#equation.
    def enterEquation(self, ctx:ModelicaParser.EquationContext):
        pass

    # Exit a parse tree produced by ModelicaParser#equation.
    def exitEquation(self, ctx:ModelicaParser.EquationContext):
        pass


    # Enter a parse tree produced by ModelicaParser#statement_component_reference.
    def enterStatement_component_reference(self, ctx:ModelicaParser.Statement_component_referenceContext):
        pass

    # Exit a parse tree produced by ModelicaParser#statement_component_reference.
    def exitStatement_component_reference(self, ctx:ModelicaParser.Statement_component_referenceContext):
        pass


    # Enter a parse tree produced by ModelicaParser#statement_component_function.
    def enterStatement_component_function(self, ctx:ModelicaParser.Statement_component_functionContext):
        pass

    # Exit a parse tree produced by ModelicaParser#statement_component_function.
    def exitStatement_component_function(self, ctx:ModelicaParser.Statement_component_functionContext):
        pass


    # Enter a parse tree produced by ModelicaParser#statement_break.
    def enterStatement_break(self, ctx:ModelicaParser.Statement_breakContext):
        pass

    # Exit a parse tree produced by ModelicaParser#statement_break.
    def exitStatement_break(self, ctx:ModelicaParser.Statement_breakContext):
        pass


    # Enter a parse tree produced by ModelicaParser#statement_return.
    def enterStatement_return(self, ctx:ModelicaParser.Statement_returnContext):
        pass

    # Exit a parse tree produced by ModelicaParser#statement_return.
    def exitStatement_return(self, ctx:ModelicaParser.Statement_returnContext):
        pass


    # Enter a parse tree produced by ModelicaParser#statement_if.
    def enterStatement_if(self, ctx:ModelicaParser.Statement_ifContext):
        pass

    # Exit a parse tree produced by ModelicaParser#statement_if.
    def exitStatement_if(self, ctx:ModelicaParser.Statement_ifContext):
        pass


    # Enter a parse tree produced by ModelicaParser#statement_for.
    def enterStatement_for(self, ctx:ModelicaParser.Statement_forContext):
        pass

    # Exit a parse tree produced by ModelicaParser#statement_for.
    def exitStatement_for(self, ctx:ModelicaParser.Statement_forContext):
        pass


    # Enter a parse tree produced by ModelicaParser#statement_while.
    def enterStatement_while(self, ctx:ModelicaParser.Statement_whileContext):
        pass

    # Exit a parse tree produced by ModelicaParser#statement_while.
    def exitStatement_while(self, ctx:ModelicaParser.Statement_whileContext):
        pass


    # Enter a parse tree produced by ModelicaParser#statement_when.
    def enterStatement_when(self, ctx:ModelicaParser.Statement_whenContext):
        pass

    # Exit a parse tree produced by ModelicaParser#statement_when.
    def exitStatement_when(self, ctx:ModelicaParser.Statement_whenContext):
        pass


    # Enter a parse tree produced by ModelicaParser#statement.
    def enterStatement(self, ctx:ModelicaParser.StatementContext):
        pass

    # Exit a parse tree produced by ModelicaParser#statement.
    def exitStatement(self, ctx:ModelicaParser.StatementContext):
        pass


    # Enter a parse tree produced by ModelicaParser#if_equation.
    def enterIf_equation(self, ctx:ModelicaParser.If_equationContext):
        pass

    # Exit a parse tree produced by ModelicaParser#if_equation.
    def exitIf_equation(self, ctx:ModelicaParser.If_equationContext):
        pass


    # Enter a parse tree produced by ModelicaParser#if_statement.
    def enterIf_statement(self, ctx:ModelicaParser.If_statementContext):
        pass

    # Exit a parse tree produced by ModelicaParser#if_statement.
    def exitIf_statement(self, ctx:ModelicaParser.If_statementContext):
        pass


    # Enter a parse tree produced by ModelicaParser#for_equation.
    def enterFor_equation(self, ctx:ModelicaParser.For_equationContext):
        pass

    # Exit a parse tree produced by ModelicaParser#for_equation.
    def exitFor_equation(self, ctx:ModelicaParser.For_equationContext):
        pass


    # Enter a parse tree produced by ModelicaParser#for_statement.
    def enterFor_statement(self, ctx:ModelicaParser.For_statementContext):
        pass

    # Exit a parse tree produced by ModelicaParser#for_statement.
    def exitFor_statement(self, ctx:ModelicaParser.For_statementContext):
        pass


    # Enter a parse tree produced by ModelicaParser#for_indices.
    def enterFor_indices(self, ctx:ModelicaParser.For_indicesContext):
        pass

    # Exit a parse tree produced by ModelicaParser#for_indices.
    def exitFor_indices(self, ctx:ModelicaParser.For_indicesContext):
        pass


    # Enter a parse tree produced by ModelicaParser#for_index.
    def enterFor_index(self, ctx:ModelicaParser.For_indexContext):
        pass

    # Exit a parse tree produced by ModelicaParser#for_index.
    def exitFor_index(self, ctx:ModelicaParser.For_indexContext):
        pass


    # Enter a parse tree produced by ModelicaParser#while_statement.
    def enterWhile_statement(self, ctx:ModelicaParser.While_statementContext):
        pass

    # Exit a parse tree produced by ModelicaParser#while_statement.
    def exitWhile_statement(self, ctx:ModelicaParser.While_statementContext):
        pass


    # Enter a parse tree produced by ModelicaParser#when_equation.
    def enterWhen_equation(self, ctx:ModelicaParser.When_equationContext):
        pass

    # Exit a parse tree produced by ModelicaParser#when_equation.
    def exitWhen_equation(self, ctx:ModelicaParser.When_equationContext):
        pass


    # Enter a parse tree produced by ModelicaParser#when_statement.
    def enterWhen_statement(self, ctx:ModelicaParser.When_statementContext):
        pass

    # Exit a parse tree produced by ModelicaParser#when_statement.
    def exitWhen_statement(self, ctx:ModelicaParser.When_statementContext):
        pass


    # Enter a parse tree produced by ModelicaParser#connect_clause.
    def enterConnect_clause(self, ctx:ModelicaParser.Connect_clauseContext):
        pass

    # Exit a parse tree produced by ModelicaParser#connect_clause.
    def exitConnect_clause(self, ctx:ModelicaParser.Connect_clauseContext):
        pass


    # Enter a parse tree produced by ModelicaParser#expression_simple.
    def enterExpression_simple(self, ctx:ModelicaParser.Expression_simpleContext):
        pass

    # Exit a parse tree produced by ModelicaParser#expression_simple.
    def exitExpression_simple(self, ctx:ModelicaParser.Expression_simpleContext):
        pass


    # Enter a parse tree produced by ModelicaParser#expression_if.
    def enterExpression_if(self, ctx:ModelicaParser.Expression_ifContext):
        pass

    # Exit a parse tree produced by ModelicaParser#expression_if.
    def exitExpression_if(self, ctx:ModelicaParser.Expression_ifContext):
        pass


    # Enter a parse tree produced by ModelicaParser#simple_expression.
    def enterSimple_expression(self, ctx:ModelicaParser.Simple_expressionContext):
        pass

    # Exit a parse tree produced by ModelicaParser#simple_expression.
    def exitSimple_expression(self, ctx:ModelicaParser.Simple_expressionContext):
        pass


    # Enter a parse tree produced by ModelicaParser#expr_neg.
    def enterExpr_neg(self, ctx:ModelicaParser.Expr_negContext):
        pass

    # Exit a parse tree produced by ModelicaParser#expr_neg.
    def exitExpr_neg(self, ctx:ModelicaParser.Expr_negContext):
        pass


    # Enter a parse tree produced by ModelicaParser#expr_add.
    def enterExpr_add(self, ctx:ModelicaParser.Expr_addContext):
        pass

    # Exit a parse tree produced by ModelicaParser#expr_add.
    def exitExpr_add(self, ctx:ModelicaParser.Expr_addContext):
        pass


    # Enter a parse tree produced by ModelicaParser#expr_exp.
    def enterExpr_exp(self, ctx:ModelicaParser.Expr_expContext):
        pass

    # Exit a parse tree produced by ModelicaParser#expr_exp.
    def exitExpr_exp(self, ctx:ModelicaParser.Expr_expContext):
        pass


    # Enter a parse tree produced by ModelicaParser#expr_or.
    def enterExpr_or(self, ctx:ModelicaParser.Expr_orContext):
        pass

    # Exit a parse tree produced by ModelicaParser#expr_or.
    def exitExpr_or(self, ctx:ModelicaParser.Expr_orContext):
        pass


    # Enter a parse tree produced by ModelicaParser#expr_primary.
    def enterExpr_primary(self, ctx:ModelicaParser.Expr_primaryContext):
        pass

    # Exit a parse tree produced by ModelicaParser#expr_primary.
    def exitExpr_primary(self, ctx:ModelicaParser.Expr_primaryContext):
        pass


    # Enter a parse tree produced by ModelicaParser#expr_and.
    def enterExpr_and(self, ctx:ModelicaParser.Expr_andContext):
        pass

    # Exit a parse tree produced by ModelicaParser#expr_and.
    def exitExpr_and(self, ctx:ModelicaParser.Expr_andContext):
        pass


    # Enter a parse tree produced by ModelicaParser#expr_rel.
    def enterExpr_rel(self, ctx:ModelicaParser.Expr_relContext):
        pass

    # Exit a parse tree produced by ModelicaParser#expr_rel.
    def exitExpr_rel(self, ctx:ModelicaParser.Expr_relContext):
        pass


    # Enter a parse tree produced by ModelicaParser#expr_not.
    def enterExpr_not(self, ctx:ModelicaParser.Expr_notContext):
        pass

    # Exit a parse tree produced by ModelicaParser#expr_not.
    def exitExpr_not(self, ctx:ModelicaParser.Expr_notContext):
        pass


    # Enter a parse tree produced by ModelicaParser#expr_mul.
    def enterExpr_mul(self, ctx:ModelicaParser.Expr_mulContext):
        pass

    # Exit a parse tree produced by ModelicaParser#expr_mul.
    def exitExpr_mul(self, ctx:ModelicaParser.Expr_mulContext):
        pass


    # Enter a parse tree produced by ModelicaParser#primary_unsigned_number.
    def enterPrimary_unsigned_number(self, ctx:ModelicaParser.Primary_unsigned_numberContext):
        pass

    # Exit a parse tree produced by ModelicaParser#primary_unsigned_number.
    def exitPrimary_unsigned_number(self, ctx:ModelicaParser.Primary_unsigned_numberContext):
        pass


    # Enter a parse tree produced by ModelicaParser#primary_string.
    def enterPrimary_string(self, ctx:ModelicaParser.Primary_stringContext):
        pass

    # Exit a parse tree produced by ModelicaParser#primary_string.
    def exitPrimary_string(self, ctx:ModelicaParser.Primary_stringContext):
        pass


    # Enter a parse tree produced by ModelicaParser#primary_false.
    def enterPrimary_false(self, ctx:ModelicaParser.Primary_falseContext):
        pass

    # Exit a parse tree produced by ModelicaParser#primary_false.
    def exitPrimary_false(self, ctx:ModelicaParser.Primary_falseContext):
        pass


    # Enter a parse tree produced by ModelicaParser#primary_true.
    def enterPrimary_true(self, ctx:ModelicaParser.Primary_trueContext):
        pass

    # Exit a parse tree produced by ModelicaParser#primary_true.
    def exitPrimary_true(self, ctx:ModelicaParser.Primary_trueContext):
        pass


    # Enter a parse tree produced by ModelicaParser#primary_function.
    def enterPrimary_function(self, ctx:ModelicaParser.Primary_functionContext):
        pass

    # Exit a parse tree produced by ModelicaParser#primary_function.
    def exitPrimary_function(self, ctx:ModelicaParser.Primary_functionContext):
        pass


    # Enter a parse tree produced by ModelicaParser#primary_derivative.
    def enterPrimary_derivative(self, ctx:ModelicaParser.Primary_derivativeContext):
        pass

    # Exit a parse tree produced by ModelicaParser#primary_derivative.
    def exitPrimary_derivative(self, ctx:ModelicaParser.Primary_derivativeContext):
        pass


    # Enter a parse tree produced by ModelicaParser#primary_initial.
    def enterPrimary_initial(self, ctx:ModelicaParser.Primary_initialContext):
        pass

    # Exit a parse tree produced by ModelicaParser#primary_initial.
    def exitPrimary_initial(self, ctx:ModelicaParser.Primary_initialContext):
        pass


    # Enter a parse tree produced by ModelicaParser#primary_component_reference.
    def enterPrimary_component_reference(self, ctx:ModelicaParser.Primary_component_referenceContext):
        pass

    # Exit a parse tree produced by ModelicaParser#primary_component_reference.
    def exitPrimary_component_reference(self, ctx:ModelicaParser.Primary_component_referenceContext):
        pass


    # Enter a parse tree produced by ModelicaParser#primary_output_expression_list.
    def enterPrimary_output_expression_list(self, ctx:ModelicaParser.Primary_output_expression_listContext):
        pass

    # Exit a parse tree produced by ModelicaParser#primary_output_expression_list.
    def exitPrimary_output_expression_list(self, ctx:ModelicaParser.Primary_output_expression_listContext):
        pass


    # Enter a parse tree produced by ModelicaParser#primary_expression_list.
    def enterPrimary_expression_list(self, ctx:ModelicaParser.Primary_expression_listContext):
        pass

    # Exit a parse tree produced by ModelicaParser#primary_expression_list.
    def exitPrimary_expression_list(self, ctx:ModelicaParser.Primary_expression_listContext):
        pass


    # Enter a parse tree produced by ModelicaParser#primary_function_arguments.
    def enterPrimary_function_arguments(self, ctx:ModelicaParser.Primary_function_argumentsContext):
        pass

    # Exit a parse tree produced by ModelicaParser#primary_function_arguments.
    def exitPrimary_function_arguments(self, ctx:ModelicaParser.Primary_function_argumentsContext):
        pass


    # Enter a parse tree produced by ModelicaParser#primary_end.
    def enterPrimary_end(self, ctx:ModelicaParser.Primary_endContext):
        pass

    # Exit a parse tree produced by ModelicaParser#primary_end.
    def exitPrimary_end(self, ctx:ModelicaParser.Primary_endContext):
        pass


    # Enter a parse tree produced by ModelicaParser#name.
    def enterName(self, ctx:ModelicaParser.NameContext):
        pass

    # Exit a parse tree produced by ModelicaParser#name.
    def exitName(self, ctx:ModelicaParser.NameContext):
        pass


    # Enter a parse tree produced by ModelicaParser#component_reference_element.
    def enterComponent_reference_element(self, ctx:ModelicaParser.Component_reference_elementContext):
        pass

    # Exit a parse tree produced by ModelicaParser#component_reference_element.
    def exitComponent_reference_element(self, ctx:ModelicaParser.Component_reference_elementContext):
        pass


    # Enter a parse tree produced by ModelicaParser#component_reference.
    def enterComponent_reference(self, ctx:ModelicaParser.Component_referenceContext):
        pass

    # Exit a parse tree produced by ModelicaParser#component_reference.
    def exitComponent_reference(self, ctx:ModelicaParser.Component_referenceContext):
        pass


    # Enter a parse tree produced by ModelicaParser#function_call_args.
    def enterFunction_call_args(self, ctx:ModelicaParser.Function_call_argsContext):
        pass

    # Exit a parse tree produced by ModelicaParser#function_call_args.
    def exitFunction_call_args(self, ctx:ModelicaParser.Function_call_argsContext):
        pass


    # Enter a parse tree produced by ModelicaParser#function_arguments.
    def enterFunction_arguments(self, ctx:ModelicaParser.Function_argumentsContext):
        pass

    # Exit a parse tree produced by ModelicaParser#function_arguments.
    def exitFunction_arguments(self, ctx:ModelicaParser.Function_argumentsContext):
        pass


    # Enter a parse tree produced by ModelicaParser#named_arguments.
    def enterNamed_arguments(self, ctx:ModelicaParser.Named_argumentsContext):
        pass

    # Exit a parse tree produced by ModelicaParser#named_arguments.
    def exitNamed_arguments(self, ctx:ModelicaParser.Named_argumentsContext):
        pass


    # Enter a parse tree produced by ModelicaParser#named_argument.
    def enterNamed_argument(self, ctx:ModelicaParser.Named_argumentContext):
        pass

    # Exit a parse tree produced by ModelicaParser#named_argument.
    def exitNamed_argument(self, ctx:ModelicaParser.Named_argumentContext):
        pass


    # Enter a parse tree produced by ModelicaParser#argument_function.
    def enterArgument_function(self, ctx:ModelicaParser.Argument_functionContext):
        pass

    # Exit a parse tree produced by ModelicaParser#argument_function.
    def exitArgument_function(self, ctx:ModelicaParser.Argument_functionContext):
        pass


    # Enter a parse tree produced by ModelicaParser#argument_expression.
    def enterArgument_expression(self, ctx:ModelicaParser.Argument_expressionContext):
        pass

    # Exit a parse tree produced by ModelicaParser#argument_expression.
    def exitArgument_expression(self, ctx:ModelicaParser.Argument_expressionContext):
        pass


    # Enter a parse tree produced by ModelicaParser#output_expression_list.
    def enterOutput_expression_list(self, ctx:ModelicaParser.Output_expression_listContext):
        pass

    # Exit a parse tree produced by ModelicaParser#output_expression_list.
    def exitOutput_expression_list(self, ctx:ModelicaParser.Output_expression_listContext):
        pass


    # Enter a parse tree produced by ModelicaParser#expression_list.
    def enterExpression_list(self, ctx:ModelicaParser.Expression_listContext):
        pass

    # Exit a parse tree produced by ModelicaParser#expression_list.
    def exitExpression_list(self, ctx:ModelicaParser.Expression_listContext):
        pass


    # Enter a parse tree produced by ModelicaParser#array_subscripts.
    def enterArray_subscripts(self, ctx:ModelicaParser.Array_subscriptsContext):
        pass

    # Exit a parse tree produced by ModelicaParser#array_subscripts.
    def exitArray_subscripts(self, ctx:ModelicaParser.Array_subscriptsContext):
        pass


    # Enter a parse tree produced by ModelicaParser#subscript.
    def enterSubscript(self, ctx:ModelicaParser.SubscriptContext):
        pass

    # Exit a parse tree produced by ModelicaParser#subscript.
    def exitSubscript(self, ctx:ModelicaParser.SubscriptContext):
        pass


    # Enter a parse tree produced by ModelicaParser#comment.
    def enterComment(self, ctx:ModelicaParser.CommentContext):
        pass

    # Exit a parse tree produced by ModelicaParser#comment.
    def exitComment(self, ctx:ModelicaParser.CommentContext):
        pass


    # Enter a parse tree produced by ModelicaParser#string_comment.
    def enterString_comment(self, ctx:ModelicaParser.String_commentContext):
        pass

    # Exit a parse tree produced by ModelicaParser#string_comment.
    def exitString_comment(self, ctx:ModelicaParser.String_commentContext):
        pass


    # Enter a parse tree produced by ModelicaParser#annotation.
    def enterAnnotation(self, ctx:ModelicaParser.AnnotationContext):
        pass

    # Exit a parse tree produced by ModelicaParser#annotation.
    def exitAnnotation(self, ctx:ModelicaParser.AnnotationContext):
        pass


